@@ -22,38 +22,52 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <net/if.h>
 
 #include "backends.h"
 #include "farms.h"
+#include "farmaddress.h"
 #include "objects.h"
 #include "network.h"
 #include "sessions.h"
-#include "nft.h"
+#include "tools.h"
 
-static int backend_s_set_marked(struct farm *f)
+#define BACKEND_MARK_MIN			0x00000001
+#define BACKEND_MARK_MAX			0x00000FFF
+
+static int backend_gen_next_mark(void)
 {
+	struct list_head *farms = obj_get_farms();
+	struct farm *f;
 	struct backend *b;
+	int mark = BACKEND_MARK_MIN;
+	int found = 0;
 
-	syslog(LOG_DEBUG, "%s():%d: finding marked backends for %s", __FUNCTION__, __LINE__, f->name);
-
-	list_for_each_entry(b, &f->backends, list) {
-		if (b->mark != DEFAULT_MARK) {
-			f->bcks_are_marked = 1;
-			return 1;
+	for (mark = BACKEND_MARK_MIN; mark <= BACKEND_MARK_MAX; mark++) {
+		found = 0;
+		list_for_each_entry(f, farms, list) {
+			list_for_each_entry(b, &f->backends, list) {
+				if (mark == b->mark) {
+					found = 1;
+					continue;
+				}
+			}
+			if (found)
+				continue;
 		}
+
+		if (!found)
+			return mark;
 	}
 
-	f->bcks_are_marked = 0;
-	return 0;
+	return DEFAULT_MARK;
 }
 
 static struct backend * backend_create(struct farm *f, char *name)
 {
 	struct backend *b = (struct backend *)malloc(sizeof(struct backend));
 	if (!b) {
-		syslog(LOG_ERR, "Backend memory allocation error");
+		tools_printlog(LOG_ERR, "Backend memory allocation error");
 		return NULL;
 	}
 
@@ -69,15 +83,16 @@ static struct backend * backend_create(struct farm *f, char *name)
 	b->srcaddr = DEFAULT_SRCADDR;
 	b->weight = DEFAULT_WEIGHT;
 	b->priority = DEFAULT_PRIORITY;
-	b->mark = DEFAULT_MARK;
+	b->mark = backend_gen_next_mark();
 	b->estconnlimit = DEFAULT_ESTCONNLIMIT;
 	b->estconnlimit_logprefix = DEFAULT_B_ESTCONNLIMIT_LOGPREFIX;
 	b->state = DEFAULT_BACKEND_STATE;
 	b->action = DEFAULT_ACTION;
 
+	b->parent->bcks_have_port = 0;
+
 	list_add_tail(&b->list, &f->backends);
 	f->total_bcks++;
-	backend_s_set_marked(f);
 
 	return b;
 }
@@ -107,17 +122,56 @@ static int backend_delete_node(struct backend *b)
 	return 0;
 }
 
-static int backend_delete(struct backend *b)
+static int backend_below_prio(struct backend *b)
 {
 	struct farm *f = b->parent;
-	backend_set_action(b, ACTION_STOP);
 
-	if (f->priority >= 1 && b->priority <= f->priority) {
-		farm_set_priority(f, f->priority-1);
+	tools_printlog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d <= farm prio %d",
+				   __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority, f->priority);
+
+	return (b->priority <= f->priority);
+}
+
+static int backend_s_set_ports(struct farm *f)
+{
+	struct backend *b;
+
+	tools_printlog(LOG_DEBUG, "%s():%d: finding backends with port for %s", __FUNCTION__, __LINE__, f->name);
+
+	list_for_each_entry(b, &f->backends, list) {
+		if (strcmp(b->port, DEFAULT_PORT) == 0) {
+			f->bcks_have_port = 0;
+			return 0;
+		}
+	}
+
+	f->bcks_have_port = 1;
+	return 1;
+}
+
+static int backend_delete(struct backend *b)
+{
+	if (!b)
+		return 0;
+
+	struct farm *f = b->parent;
+	backend_set_action(b, ACTION_STOP);
+	session_backend_action(f, b, ACTION_STOP);
+
+	if (backend_below_prio(b)) {
+		backend_s_gen_priority(f, ACTION_DELETE);
 		obj_rulerize(OBJ_START);
 	}
 
+	session_backend_action(f, b, ACTION_DELETE);
 	backend_delete_node(b);
+	backend_s_set_ports(f);
+
+	if (backend_s_gen_priority(f, ACTION_DELETE)) {
+		farm_set_action(f, ACTION_RELOAD);
+		farmaddress_s_set_action(f, ACTION_RELOAD);
+		obj_rulerize(OBJ_START);
+	}
 
 	return 0;
 }
@@ -127,47 +181,46 @@ void backend_s_print(struct farm *f)
 	struct backend *b;
 
 	list_for_each_entry(b, &f->backends, list) {
-		syslog(LOG_DEBUG,"    [backend] ");
-		syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_NAME, b->name);
+		tools_printlog(LOG_DEBUG,"    [backend] ");
+		tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_NAME, b->name);
 
 		if (b->fqdn)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_FQDN, b->fqdn);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_FQDN, b->fqdn);
 
 		if (b->oface)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_OFACE, b->oface);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_OFACE, b->oface);
 
-		syslog(LOG_DEBUG,"      *[ofidx] %d", b->ofidx);
+		tools_printlog(LOG_DEBUG,"      *[ofidx] %d", b->ofidx);
 
 		if (b->ipaddr)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_IPADDR, b->ipaddr);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_IPADDR, b->ipaddr);
 
 		if (b->ethaddr)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_ETHADDR, b->ethaddr);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_ETHADDR, b->ethaddr);
 
 		if (b->port)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_PORT, b->port);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_PORT, b->port);
 
 		if (b->srcaddr)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_SRCADDR, b->srcaddr);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_SRCADDR, b->srcaddr);
 
-		syslog(LOG_DEBUG,"       [%s] 0x%x", CONFIG_KEY_MARK, b->mark);
-		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_ESTCONNLIMIT, b->estconnlimit);
+		tools_printlog(LOG_DEBUG,"       [%s] 0x%x", CONFIG_KEY_MARK, b->mark);
+		tools_printlog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_ESTCONNLIMIT, b->estconnlimit);
 		if (b->estconnlimit_logprefix && strcmp(b->estconnlimit_logprefix, DEFAULT_B_ESTCONNLIMIT_LOGPREFIX) != 0)
-			syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_ESTCONNLIMIT_LOGPREFIX, b->estconnlimit_logprefix);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_ESTCONNLIMIT_LOGPREFIX, b->estconnlimit_logprefix);
 
-		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_WEIGHT, b->weight);
-		syslog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_PRIORITY, b->priority);
-		syslog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_STATE, obj_print_state(b->state));
-		syslog(LOG_DEBUG,"      *[%s] %d", CONFIG_KEY_ACTION, b->action);
+		tools_printlog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_WEIGHT, b->weight);
+		tools_printlog(LOG_DEBUG,"       [%s] %d", CONFIG_KEY_PRIORITY, b->priority);
+		tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_STATE, obj_print_state(b->state));
+		tools_printlog(LOG_DEBUG,"      *[%s] %d", CONFIG_KEY_ACTION, b->action);
 	}
 }
 
 struct backend * backend_lookup_by_key(struct farm *f, int key, const char *name, int value)
 {
 	struct backend *b;
-	int bckmark;
 
-	syslog(LOG_DEBUG, "%s():%d: farm %s key %d name %s value %d", __FUNCTION__, __LINE__, f->name, key, name, value);
+	tools_printlog(LOG_DEBUG, "%s():%d: farm %s key %d name %s value %d", __FUNCTION__, __LINE__, f->name, key, name, value);
 
 	list_for_each_entry(b, &f->backends, list) {
 		switch (key) {
@@ -176,8 +229,7 @@ struct backend * backend_lookup_by_key(struct farm *f, int key, const char *name
 				return b;
 			break;
 		case KEY_MARK:
-			bckmark = get_bck_mark(b);
-			if (bckmark == value)
+			if (value == backend_get_mark(b))
 				return b;
 			break;
 		case KEY_ETHADDR:
@@ -199,6 +251,8 @@ struct backend * backend_lookup_by_key(struct farm *f, int key, const char *name
 static int backend_set_ipaddr_from_ether(struct backend *b)
 {
 	struct farm *f = b->parent;
+	struct farmaddress *fa;
+	struct address *a;
 	int ret = -1;
 	unsigned char dst_ethaddr[ETH_HW_ADDR_LEN];
 	unsigned char src_ethaddr[ETH_HW_ADDR_LEN];
@@ -206,23 +260,31 @@ static int backend_set_ipaddr_from_ether(struct backend *b)
 	int *oface;
 	char **source_ip;
 
-	if (!farm_is_ingress_mode(f))
+	if (!farm_is_ingress_mode(f) || (f->state != VALUE_STATE_UP && f->state != VALUE_STATE_CONFERR))
 		return 0;
 
-	if (f->iethaddr == DEFAULT_ETHADDR ||
+	fa = farmaddress_get_first(f);
+	if (!fa) {
+		tools_printlog(LOG_INFO, "%s():%d: no farm address configured in %s", __FUNCTION__, __LINE__, f->name);
+		return -1;
+	}
+
+	a = fa->address;
+	if (a->iethaddr == DEFAULT_ETHADDR ||
 		b->ipaddr == DEFAULT_IPADDR ||
-		f->ofidx == DEFAULT_IFIDX)
+		fa->farm->ofidx == DEFAULT_IFIDX)
 		return -1;
 
-	sscanf(f->iethaddr, "%hhx:%hhx:%hhx:%hhx:%hhx:%hhx", &src_ethaddr[0], &src_ethaddr[1], &src_ethaddr[2], &src_ethaddr[3], &src_ethaddr[4], &src_ethaddr[5]);
+	sscanf(a->iethaddr, "%hhx:%hhx:%hhx:%hhx:%hhx:%hhx", &src_ethaddr[0], &src_ethaddr[1], &src_ethaddr[2], &src_ethaddr[3], &src_ethaddr[4], &src_ethaddr[5]);
 
-	oface = &f->ifidx;
-	source_ip = &f->virtaddr;
+	oface = &a->ifidx;
+
+	source_ip = &(a->ipaddr);
 
 	if (f->srcaddr != DEFAULT_SRCADDR)
 		source_ip = &f->srcaddr;
 
-	ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, f->family, *source_ip, b->ipaddr, *oface);
+	ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, a->family, *source_ip, b->ipaddr, *oface);
 
 	if (ret != 0) {
 		oface = &f->ofidx;
@@ -236,15 +298,17 @@ static int backend_set_ipaddr_from_ether(struct backend *b)
 		if (b->srcaddr != DEFAULT_SRCADDR)
 			source_ip = &b->srcaddr;
 
-		ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, f->family, *source_ip, b->ipaddr, *oface);
+		ret = net_get_neigh_ether((unsigned char **) &dst_ethaddr, src_ethaddr, a->family, *source_ip, b->ipaddr, *oface);
 	}
 
 	if (ret == 0) {
 		sprintf(streth, "%02x:%02x:%02x:%02x:%02x:%02x", dst_ethaddr[0],
 			dst_ethaddr[1], dst_ethaddr[2], dst_ethaddr[3], dst_ethaddr[4], dst_ethaddr[5]);
 
-		syslog(LOG_DEBUG, "%s():%d: discovered ether address for %s is %s", __FUNCTION__, __LINE__, b->name, streth);
+		tools_printlog(LOG_DEBUG, "%s():%d: discovered ether address for %s is %s", __FUNCTION__, __LINE__, b->name, streth);
 
+		if (b->ethaddr)
+			free(b->ethaddr);
 		obj_set_attribute_string(streth, &b->ethaddr);
 	}
 
@@ -256,8 +320,8 @@ static int backend_set_weight(struct backend *b, int new_value)
 	struct farm *f = b->parent;
 	int old_value = b->weight;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
 	b->weight = new_value;
 
@@ -271,8 +335,8 @@ static int backend_set_estconnlimit(struct backend *b, int new_value)
 {
 	int old_value = b->estconnlimit;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
 	if (new_value == old_value)
 		return 0;
@@ -286,9 +350,10 @@ static void backend_s_update_counters(struct farm *f)
 {
 	struct backend *bp, *next;
 
-	syslog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
+	tools_printlog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
 
 	f->bcks_available = 0;
+	f->bcks_usable = 0;
 	f->total_weight = 0;
 
 	list_for_each_entry_safe(bp, next, &f->backends, list) {
@@ -296,6 +361,8 @@ static void backend_s_update_counters(struct farm *f)
 			f->bcks_available++;
 			f->total_weight += bp->weight;
 		}
+		if (backend_is_usable(bp))
+			f->bcks_usable++;
 	}
 }
 
@@ -303,39 +370,23 @@ static int backend_set_priority(struct backend *b, int new_value)
 {
 	int old_value = b->priority;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
 	if (new_value <= 0)
 		return -1;
 
 	b->priority = new_value;
+	backend_s_gen_priority(b->parent, ACTION_RELOAD);
 
 	return 0;
 }
 
-static int backend_s_set_ports(struct farm *f)
-{
-	struct backend *b;
-
-	syslog(LOG_DEBUG, "%s():%d: finding backends with port for %s", __FUNCTION__, __LINE__, f->name);
-
-	list_for_each_entry(b, &f->backends, list) {
-		if (strcmp(b->port, DEFAULT_PORT) != 0) {
-			f->bcks_have_port = 1;
-			return 1;
-		}
-	}
-
-	f->bcks_have_port = 0;
-	return 0;
-}
-
 static int backend_s_set_srcaddr(struct farm *f)
 {
 	struct backend *b;
 
-	syslog(LOG_DEBUG, "%s():%d: finding backends with srouce address for %s", __FUNCTION__, __LINE__, f->name);
+	tools_printlog(LOG_DEBUG, "%s():%d: finding backends with srouce address for %s", __FUNCTION__, __LINE__, f->name);
 
 	list_for_each_entry(b, &f->backends, list) {
 		if (b->srcaddr && strcmp(b->srcaddr, "") != 0) {
@@ -352,16 +403,14 @@ static int backend_set_mark(struct backend *b, int new_value)
 {
 	int old_value = b->mark;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	if (new_value < BACKEND_MARK_MIN || new_value > BACKEND_MARK_MAX)
+		return 0;
+
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %d, but new value will be %d",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
 	b->mark = new_value;
 
-	if (b->mark != DEFAULT_MARK)
-		b->parent->bcks_are_marked = 1;
-	else
-		backend_s_set_marked(b->parent);
-
 	return 0;
 }
 
@@ -369,14 +418,17 @@ static int backend_set_port(struct backend *b, char *new_value)
 {
 	char *old_value = b->port;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
-	       __FUNCTION__, __LINE__, old_value, new_value);
-
-	obj_set_attribute_string(new_value, &b->port);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
 	if (strcmp(b->port, DEFAULT_PORT) != 0)
-		b->parent->bcks_have_port = 1;
-	else
+		free(b->port);
+	obj_set_attribute_string(new_value, &b->port);
+
+	if (b->parent->bcks_have_port) {
+		if (strcmp(b->port, DEFAULT_PORT) == 0)
+			b->parent->bcks_have_port = 0;
+	} else
 		backend_s_set_ports(b->parent);
 
 	return 0;
@@ -386,9 +438,11 @@ static int backend_set_srcaddr(struct backend *b, char *new_value)
 {
 	char *old_value = b->srcaddr;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
+	if (b->srcaddr)
+		free(b->srcaddr);
 	obj_set_attribute_string(new_value, &b->srcaddr);
 
 	if (b->srcaddr && strcmp(b->srcaddr, "") != 0)
@@ -399,54 +453,58 @@ static int backend_set_srcaddr(struct backend *b, char *new_value)
 	return 0;
 }
 
-static int backend_set_ifinfo(struct backend *b, int key)
+static int backend_set_ifinfo(struct backend *b)
 {
 	struct farm *f = b->parent;
 	char if_str[IFNAMSIZ];
 	int if_index;
 	int ret = 0;
 
-	syslog(LOG_DEBUG, "%s():%d: backend %s set interface info for interface key %d", __FUNCTION__, __LINE__, b->name, key);
+	tools_printlog(LOG_DEBUG, "%s():%d: backend %s set interface info", __FUNCTION__, __LINE__, b->name);
 
-	if (!farm_is_ingress_mode(f)) {
-		syslog(LOG_DEBUG, "%s():%d: farm %s is not in ingress mode", __FUNCTION__, __LINE__, f->name);
+	if (!farm_is_ingress_mode(f) || (f->state != VALUE_STATE_UP && f->state != VALUE_STATE_CONFERR))
 		return 0;
-	}
 
 	if (f->oface && strcmp(f->oface, IFACE_LOOPBACK) == 0) {
-		syslog(LOG_DEBUG, "%s():%d: backend %s in farm %s doesn't require output netinfo, loopback interface", __FUNCTION__, __LINE__, b->name, f->name);
-		f->ifidx = 0;
+		tools_printlog(LOG_DEBUG, "%s():%d: backend %s in farm %s doesn't require output netinfo, loopback interface", __FUNCTION__, __LINE__, b->name, f->name);
+		f->ofidx = 0;
 		return 0;
 	}
 
-	switch (key) {
-	case KEY_OFACE:
-		if (!b || b->ipaddr == DEFAULT_IPADDR) {
-			syslog(LOG_ERR, "%s():%d: there is no backend yet in the farm %s", __FUNCTION__, __LINE__, f->name);
-			return 0;
-		}
+	if (!b || b->ipaddr == DEFAULT_IPADDR) {
+		tools_printlog(LOG_ERR, "%s():%d: there is no backend yet in the farm %s", __FUNCTION__, __LINE__, f->name);
+		return 0;
+	}
 
-		ret = net_get_local_ifidx_per_remote_host(b->ipaddr, &if_index);
-		if (ret == -1) {
-			syslog(LOG_ERR, "%s():%d: unable to get the outbound interface to %s for the backend %s in farm %s", __FUNCTION__, __LINE__, b->ipaddr, b->name, f->name);
-			return -1;
-		}
+	ret = net_get_local_ifidx_per_remote_host(b->ipaddr, &if_index);
+	if (ret == -1) {
+		tools_printlog(LOG_ERR, "%s():%d: unable to get the outbound interface to %s for the backend %s in farm %s", __FUNCTION__, __LINE__, b->ipaddr, b->name, f->name);
+		return -1;
+	}
 
-		if (f->ofidx != if_index) {
-			f->bcks_have_if = 1;
-			b->ofidx = if_index;
-		}
+	if (f->ofidx == -1)
+		f->ofidx = if_index;
 
-		if (if_indextoname(if_index, if_str) == NULL) {
-			syslog(LOG_ERR, "%s():%d: unable to get the outbound interface name with index %d required by the backend %s in farm %s", __FUNCTION__, __LINE__, if_index, b->name, f->name);
-			return -1;
-		}
+	if (f->ofidx != if_index) {
+		f->bcks_have_if = 1;
+		b->ofidx = if_index;
+	}
 
-		if (strcmp(f->oface, if_str) != 0) {
-			obj_set_attribute_string(if_str, &b->oface);
-			net_strim_netface(b->oface);
-		}
-		break;
+	if (if_indextoname(if_index, if_str) == NULL) {
+		tools_printlog(LOG_ERR, "%s():%d: unable to get the outbound interface name with index %d required by the backend %s in farm %s", __FUNCTION__, __LINE__, if_index, b->name, f->name);
+		return -1;
+	}
+
+	if (!f->oface) {
+		obj_set_attribute_string(if_str, &f->oface);
+		net_strim_netface(f->oface);
+		return 0;
+	}
+
+	if (f->oface && strcmp(f->oface, if_str) != 0) {
+		free(b->oface);
+		obj_set_attribute_string(if_str, &b->oface);
+		net_strim_netface(b->oface);
 	}
 
 	return 0;
@@ -455,36 +513,45 @@ static int backend_set_ifinfo(struct backend *b, int key)
 static int backend_set_ipaddr(struct backend *b, char *new_value)
 {
 	char *old_value = b->ipaddr;
+	int netconfig;
 
-	syslog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
-	       __FUNCTION__, __LINE__, old_value, new_value);
+	tools_printlog(LOG_DEBUG, "%s():%d: current value is %s, but new value will be %s",
+				   __FUNCTION__, __LINE__, old_value, new_value);
 
+	if (b->ipaddr)
+		free(b->ipaddr);
+	if (b->ethaddr)
+		free(b->ethaddr);
 	obj_set_attribute_string(new_value, &b->ipaddr);
 	obj_set_attribute_string("", &b->ethaddr);
 
-	if (farm_set_ifinfo(b->parent, KEY_OFACE) == 0 &&
-		backend_set_ifinfo(b, KEY_OFACE) == 0 &&
-	    backend_set_ipaddr_from_ether(b) == 0 ) {
-		if (old_value != DEFAULT_IPADDR && b->state == VALUE_STATE_CONFERR)
+	netconfig = (backend_set_ifinfo(b) == 0 && backend_set_ipaddr_from_ether(b) == 0);
+
+	if (old_value == DEFAULT_IPADDR)
+		return 0;
+
+	if (netconfig) {
+		if (b->state == VALUE_STATE_CONFERR)
 			backend_set_state(b, VALUE_STATE_UP);
-	} else {
-		syslog(LOG_DEBUG, "%s():%d: backend %s comes to OFF", __FUNCTION__, __LINE__, b->name);
-		if (old_value != DEFAULT_IPADDR)
-			backend_set_state(b, VALUE_STATE_CONFERR);
-	}
+	} else
+		backend_set_state(b, VALUE_STATE_CONFERR);
 
 	return 0;
 }
 
-static int backend_is_usable(struct backend *b)
+int backend_is_usable(struct backend *b)
 {
-	struct farm *f = b->parent;
+	tools_printlog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
+				   __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
 
-	syslog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
+	return ((b->state == VALUE_STATE_UP || b->state == VALUE_STATE_OFF) && backend_below_prio(b));
+}
 
-	return (b->state == VALUE_STATE_UP) &&
-			(b->priority <= f->priority);
+int backend_no_port(struct backend *b)
+{
+	if (obj_equ_attribute_string(b->port, DEFAULT_PORT))
+		return 1;
+	return 0;
 }
 
 int backend_changed(struct config_pair *c)
@@ -495,7 +562,7 @@ int backend_changed(struct config_pair *c)
 	if (!f || !b)
 		return -1;
 
-	syslog(LOG_DEBUG, "%s():%d: farm %s backend %s with param %d", __FUNCTION__, __LINE__, f->name, b->name, c->key);
+	tools_printlog(LOG_DEBUG, "%s():%d: farm %s backend %s with param %d", __FUNCTION__, __LINE__, f->name, b->name, c->key);
 
 	switch (c->key) {
 	case KEY_NAME:
@@ -551,8 +618,8 @@ int backend_validate(struct backend *b)
 {
 	struct farm *f = b->parent;
 
-	syslog(LOG_DEBUG, "%s():%d: validating backend %s of farm %s",
-	       __FUNCTION__, __LINE__, b->name, f->name);
+	tools_printlog(LOG_DEBUG, "%s():%d: validating backend %s of farm %s",
+				   __FUNCTION__, __LINE__, b->name, f->name);
 
 	if (farm_is_ingress_mode(f) &&
 		(!b->ethaddr || strcmp(b->ethaddr, "") == 0))
@@ -566,19 +633,18 @@ int backend_validate(struct backend *b)
 
 int backend_is_available(struct backend *b)
 {
-	syslog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
+	tools_printlog(LOG_DEBUG, "%s():%d: backend %s state is %s and priority %d",
+				   __FUNCTION__, __LINE__, b->name, obj_print_state(b->state), b->priority);
 
-	return (backend_is_usable(b) &&
-			backend_validate(b));
+	return (backend_validate(b) && (b->state == VALUE_STATE_UP) && backend_below_prio(b));
 }
 
 int backend_set_action(struct backend *b, int action)
 {
 	int is_actionated = 0;
 
-	syslog(LOG_DEBUG, "%s():%d: bck %s action %d state %d - new action %d",
-	       __FUNCTION__, __LINE__, b->name, b->action, b->state, action);
+	tools_printlog(LOG_DEBUG, "%s():%d: bck %s action %d state %d - new action %d",
+				   __FUNCTION__, __LINE__, b->name, b->action, b->state, action);
 
 	if (action == ACTION_DELETE) {
 		backend_delete(b);
@@ -586,19 +652,17 @@ int backend_set_action(struct backend *b, int action)
 	}
 
 	if (action == ACTION_STOP) {
-		if (backend_is_available(b))
-		{
+		if (backend_is_available(b)) {
 			b->action = action;
 			is_actionated = 1;
 		}
-		backend_set_state(b, VALUE_STATE_OFF);
+		backend_set_state(b, VALUE_STATE_DOWN);
 
 		return is_actionated;
 	}
 
 	if (action == ACTION_START) {
-		if (!backend_is_available(b))
-		{
+		if (!backend_is_available(b)) {
 			b->action = action;
 			is_actionated = 1;
 		}
@@ -633,6 +697,7 @@ int backend_s_delete(struct farm *f)
 
 	f->total_bcks = 0;
 	f->bcks_available = 0;
+	f->bcks_usable = 0;
 	f->total_weight = 0;
 	f->bcks_have_if = 0;
 
@@ -672,15 +737,20 @@ int backend_set_attribute(struct config_pair *c)
 		obj_set_current_backend(b);
 		break;
 	case KEY_NEWNAME:
+		free(b->name);
 		obj_set_attribute_string(c->str_value, &b->name);
 		break;
 	case KEY_FQDN:
+		if (strcmp(b->fqdn, DEFAULT_FQDN) != 0)
+			free(b->fqdn);
 		obj_set_attribute_string(c->str_value, &b->fqdn);
 		break;
 	case KEY_IPADDR:
 		backend_set_ipaddr(b, c->str_value);
 		break;
 	case KEY_ETHADDR:
+		if (b->ethaddr)
+			free(b->ethaddr);
 		obj_set_attribute_string(c->str_value, &b->ethaddr);
 		break;
 	case KEY_PORT:
@@ -711,6 +781,8 @@ int backend_set_attribute(struct config_pair *c)
 		backend_set_action(b, c->int_value);
 		break;
 	case KEY_ESTCONNLIMIT_LOGPREFIX:
+		if (strcmp(b->estconnlimit_logprefix, DEFAULT_B_ESTCONNLIMIT_LOGPREFIX) != 0)
+			free(b->estconnlimit_logprefix);
 		obj_set_attribute_string(c->str_value, &b->estconnlimit_logprefix);
 		break;
 	default:
@@ -720,56 +792,59 @@ int backend_set_attribute(struct config_pair *c)
 	return PARSER_OK;
 }
 
-static int backend_switch(struct backend *b)
-{
-	struct farm *f = b->parent;
-
-	syslog(LOG_DEBUG, "%s():%d: backend %s with state %s switched",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(b->state));
-
-	if (f->persistence != VALUE_META_NONE)
-		session_backend_action(f, b, b->state);
-
-	if (b->state == VALUE_STATE_UP)
-		b->action = ACTION_START;
-	else
-		b->action = ACTION_STOP;
-
-	farm_set_action(f, ACTION_RELOAD);
-	backend_s_update_counters(f);
-
-	return 0;
-}
-
 int backend_set_state(struct backend *b, int new_value)
 {
 	int old_value = b->state;
+	struct farm *f = b->parent;
 
-	syslog(LOG_DEBUG, "%s():%d: backend %s current value is %s, but new value will be %s",
-	       __FUNCTION__, __LINE__, b->name, obj_print_state(old_value), obj_print_state(new_value));
+	tools_printlog(LOG_DEBUG, "%s():%d: backend %s current value is %s, but new value will be %s",
+				   __FUNCTION__, __LINE__, b->name, obj_print_state(old_value), obj_print_state(new_value));
+
+	if (new_value == VALUE_STATE_UP) {
+		if (!backend_validate(b)) {
+			new_value = VALUE_STATE_CONFERR;
+		}
+
+		if (!backend_below_prio(b))
+			new_value = VALUE_STATE_AVAIL;
+	}
 
 	if (old_value == new_value)
 		return 0;
 
+	b->state = new_value;
+
 	switch (new_value) {
+	case VALUE_STATE_CONFERR:
+	case VALUE_STATE_OFF:
+		if (old_value == VALUE_STATE_UP)
+			b->action = ACTION_STOP;
+		break;
+	case VALUE_STATE_AVAIL:
+		if (old_value == VALUE_STATE_UP)
+			b->action = ACTION_STOP;
+		return 0;
 	case VALUE_STATE_UP:
-		b->state = new_value;
-		if (!backend_validate(b)) {
-			b->state = VALUE_STATE_CONFERR;
-			return 0;
-		}
-		if (backend_is_usable(b))
-			backend_switch(b);
+		if (f->persistence != VALUE_META_NONE)
+			session_backend_action(f, b, ACTION_START);
+		if (old_value == VALUE_STATE_OFF)
+			b->action = ACTION_RELOAD;
+		else
+			b->action = ACTION_START;
+		break;
+	case VALUE_STATE_DOWN:
+		if (old_value == VALUE_STATE_UP || old_value == VALUE_STATE_OFF)
+			b->action = ACTION_STOP;
 		break;
 	default:
-		if (backend_is_usable(b)) {
-			b->state = new_value;
-			backend_switch(b);
-		} else
-			b->state = new_value;
 		break;
 	}
 
+	if (b->action != ACTION_NONE) {
+		farm_set_action(f, ACTION_RELOAD);
+		backend_s_gen_priority(f, ACTION_NONE);
+	}
+
 	return 0;
 }
 
@@ -783,34 +858,49 @@ int backend_s_set_ether_by_ipaddr(struct farm *f, const char *ip_bck, char *ethe
 		if (strcmp(b->ipaddr, ip_bck) != 0)
 			continue;
 
-		syslog(LOG_DEBUG, "%s():%d: backend with ip address %s found", __FUNCTION__, __LINE__, ip_bck);
+		tools_printlog(LOG_DEBUG, "%s():%d: backend with ip address %s found", __FUNCTION__, __LINE__, ip_bck);
 
 		if (!b->ethaddr || (b->ethaddr && strcmp(b->ethaddr, ether_bck) != 0)) {
+			if (f->persistence != VALUE_META_NONE)
+				session_get_timed(f);
+			if (b->ethaddr)
+				free(b->ethaddr);
 			obj_set_attribute_string(ether_bck, &b->ethaddr);
-			backend_set_state(b, VALUE_STATE_UP);
 			changed = 1;
-			syslog(LOG_INFO, "%s():%d: ether address changed for backend %s with %s", __FUNCTION__, __LINE__, b->name, ether_bck);
+			if (f->persistence != VALUE_META_NONE) {
+				session_backend_action(f, b, ACTION_RELOAD);
+				farm_set_action(f, ACTION_RELOAD);
+				obj_rulerize(OBJ_START);
+				session_s_delete(f, SESSION_TYPE_TIMED);
+			}
+
+			tools_printlog(LOG_INFO, "%s():%d: ether address changed for backend %s with %s", __FUNCTION__, __LINE__, b->name, ether_bck);
 		}
 	}
 
 	return changed;
 }
 
-int backend_s_find_ethers(struct farm *f)
+static void backend_set_netinfo(struct backend *b)
+{
+	if (backend_set_ifinfo(b) == 0 && backend_set_ipaddr_from_ether(b) == 0) {
+		if (b->state == VALUE_STATE_CONFERR)
+			backend_set_state(b, VALUE_STATE_UP);
+	} else
+		backend_set_state(b, VALUE_STATE_CONFERR);
+}
+
+int backend_s_set_netinfo(struct farm *f)
 {
 	struct backend *b;
 	int changed = 0;
 
-	syslog(LOG_DEBUG, "%s():%d: finding backends for %s", __FUNCTION__, __LINE__, f->name);
+	tools_printlog(LOG_DEBUG, "%s():%d: finding backends for %s", __FUNCTION__, __LINE__, f->name);
 
 	list_for_each_entry(b, &f->backends, list) {
-		if (!backend_is_usable(b) || backend_validate(b))
+		if (backend_validate(b))
 			continue;
-
-		if (backend_set_ipaddr_from_ether(b) == -1)
-			backend_set_state(b, VALUE_STATE_CONFERR);
-		else
-			backend_set_state(b, VALUE_STATE_UP);
+		backend_set_netinfo(b);
 	}
 
 	return changed;
@@ -832,11 +922,12 @@ int bck_pre_actionable(struct config_pair *c)
 	if (!f || !b)
 		return -1;
 
-	syslog(LOG_DEBUG, "%s():%d: pre actionable backend %s of farm %s with param %d", __FUNCTION__, __LINE__, b->name, f->name, c->key);
+	tools_printlog(LOG_DEBUG, "%s():%d: pre actionable backend %s of farm %s with param %d", __FUNCTION__, __LINE__, b->name, f->name, c->key);
 
 	// changing priority of a down backend could affect others, force a farm restart
 	if (b->state != VALUE_STATE_UP && b->state != VALUE_STATE_CONFERR && c->key == KEY_PRIORITY) {
 		farm_set_action(f, ACTION_STOP);
+		farmaddress_s_set_action(f, ACTION_STOP);
 		farm_rulerize(f);
 		return ACTION_FLUSH;
 	}
@@ -850,18 +941,19 @@ int bck_pre_actionable(struct config_pair *c)
 
 	case KEY_ETHADDR:
 	case KEY_IPADDR:
-	case KEY_PORT:
 	case KEY_SRCADDR:
 	case KEY_MARK:
 	case KEY_PRIORITY:
 	case KEY_ESTCONNLIMIT:
 		if (backend_set_action(b, ACTION_STOP)) {
 			farm_set_action(f, ACTION_RELOAD);
+			farmaddress_s_set_action(f, ACTION_RELOAD);
 			farm_rulerize(f);
 		}
 		return ACTION_START;
 		break;
 
+	case KEY_PORT:
 	case KEY_STATE:
 	case KEY_WEIGHT:
 	case KEY_ESTCONNLIMIT_LOGPREFIX:
@@ -883,12 +975,15 @@ int bck_pos_actionable(struct config_pair *c, int action)
 	if (!f || !b)
 		return -1;
 
-	syslog(LOG_DEBUG, "%s():%d: pos actionable backend %s of farm %s with param %d action %d", __FUNCTION__, __LINE__, b->name, f->name, c->key, action);
+	tools_printlog(LOG_DEBUG, "%s():%d: pos actionable backend %s of farm %s with param %d action %d", __FUNCTION__, __LINE__, b->name, f->name, c->key, action);
 
 	switch (action) {
 	case ACTION_START:
-		if (backend_set_action(b, ACTION_START))
+		if (backend_set_action(b, ACTION_START)) {
 			farm_set_action(f, ACTION_RELOAD);
+			farmaddress_s_set_action(f, ACTION_RELOAD);
+			farm_rulerize(f);
+		}
 		break;
 	case ACTION_RELOAD:
 		farm_set_action(f, ACTION_RELOAD);
@@ -904,31 +999,55 @@ int bck_pos_actionable(struct config_pair *c, int action)
 	return 0;
 }
 
-int backend_s_gen_priority(struct farm *f)
+int backend_s_gen_priority(struct farm *f, int action)
 {
 	struct backend *b, *next;
 	int are_down;
 	int old_prio = f->priority;
 	int new_prio = DEFAULT_PRIORITY;
 
-	syslog(LOG_DEBUG, "%s():%d: farm %s", __FUNCTION__, __LINE__, f->name);
-
 	do {
 		are_down = 0;
-		list_for_each_entry_safe(b, next, &f->backends, list) {
-			if (b->priority == new_prio && b->state != VALUE_STATE_UP)
+		list_for_each_entry_safe(b, next, &f->backends, list)
+			if (b->priority == new_prio && b->state != VALUE_STATE_UP && b->state != VALUE_STATE_AVAIL)
 				are_down++;
-			if (b->priority > f->priority && b->priority == new_prio && b->state == VALUE_STATE_UP)
-				backend_set_action(b, ACTION_START);
-		}
 		new_prio += are_down;
 	} while (are_down);
 
 	f->priority = new_prio;
 
-	syslog(LOG_DEBUG, "%s():%d: priority is %d", __FUNCTION__, __LINE__, f->priority);
+	list_for_each_entry_safe(b, next, &f->backends, list) {
+
+		if (b->state == VALUE_STATE_UP || b->state == VALUE_STATE_AVAIL)
+			backend_set_state(b, VALUE_STATE_UP);
+	}
 
 	backend_s_update_counters(f);
-
 	return f->priority != old_prio;
 }
+
+int backend_get_mark(struct backend *b)
+{
+	int mark = b->mark;
+
+	if (b->srcaddr && strcmp(b->srcaddr, "") != 0)
+		mark |= b->parent->mark;
+	else
+		mark |= farm_get_mark(b->parent);
+
+	return mark;
+}
+
+int backend_s_check_have_iface(struct farm *f)
+{
+	struct backend *b, *next;
+
+	list_for_each_entry_safe(b, next, &f->backends, list) {
+		if (b->ofidx == DEFAULT_IFIDX)
+			continue;
+		if (f->ofidx != b->ofidx)
+			return 1;
+	}
+
+	return 0;
+}
