@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <getopt.h>
 #include <string.h>
-#include <syslog.h>
 #include <errno.h>
 #include <unistd.h>
 #include <execinfo.h>
@@ -34,6 +33,7 @@
 #include "events.h"
 #include "network.h"
 #include "nft.h"
+#include "tools.h"
 
 #define NFTLB_SERVER_MODE		0
 #define NFTLB_FG_MODE			0
@@ -41,9 +41,8 @@
 #define NFTLB_EXIT_MODE			1
 #define NFTLB_NFT_SERIALIZE		0
 
-#define NFTLB_LOGLEVEL_DEFAULT		LOG_NOTICE
-
 unsigned int serialize = NFTLB_NFT_SERIALIZE;
+int masquerade_mark = NFTLB_MASQUERADE_MARK_DEFAULT;
 
 static void print_usage(const char *prog_name)
 {
@@ -53,6 +52,7 @@ static void print_usage(const char *prog_name)
 		"Usage: %s\n"
 		"  [ -h | --help ]			Show this help\n"
 		"  [ -l <LEVEL> | --log <LEVEL> ]	Set the syslog level\n"
+		"  [ -L <OUTPUT> | --log-output <OUTPUT> ]	Set the daemon logs output\n"
 		"  [ -c <FILE> | --config <FILE> ]	Launch with the given configuration file\n"
 		"  [ -k <KEY> | --key <KEY> ]		Set the authentication key, otherwise it'll be generated\n"
 		"  [ -e | --exit ]			Don't execute the server\n"
@@ -61,12 +61,14 @@ static void print_usage(const char *prog_name)
 		"  [ -H <HOST> | --host <HOST> ]		Set the host for the listening port\n"
 		"  [ -P <PORT> | --port <PORT> ]		Set the port for the listening port\n"
 		"  [ -S | --serial ]			Serialize nft commands\n"
+		"  [ -m | --masquerade-mark ]			Set masquerade mark in hex\n"
 		, prog_name, VERSION, prog_name);
 }
 
 static const struct option options[] = {
 	{ .name = "help",	.has_arg = 0,	.val = 'h' },
 	{ .name = "log",	.has_arg = 1,	.val = 'l' },
+	{ .name = "log-output",	.has_arg = 1,	.val = 'L' },
 	{ .name = "config",	.has_arg = 1,	.val = 'c' },
 	{ .name = "key",	.has_arg = 1,	.val = 'k' },
 	{ .name = "exit",	.has_arg = 0,	.val = 'e' },
@@ -75,12 +77,13 @@ static const struct option options[] = {
 	{ .name = "host",	.has_arg = 1,	.val = 'H' },
 	{ .name = "port",	.has_arg = 1,	.val = 'P' },
 	{ .name = "serial",	.has_arg = 0,	.val = 'S' },
+	{ .name = "masquerade-mark",	.has_arg = 1,	.val = 'm' },
 	{ NULL },
 };
 
 static void nftlb_sighandler(int signo)
 {
-	syslog(LOG_INFO, "shutting down %s, bye", PACKAGE);
+	tools_printlog(LOG_INFO, "shutting down %s, bye", PACKAGE);
 	server_fini();
 	exit(EXIT_SUCCESS);
 }
@@ -88,34 +91,71 @@ static void nftlb_sighandler(int signo)
 static void nftlb_trace() {
 	void *buffer[255];
 	char **str;
-	int i;
+	int i, level;
 	const int calls = backtrace(buffer, sizeof(buffer) / sizeof(void *));
 
-	syslog(LOG_ERR, "SIGSEGV received!");
+	tools_printlog(LOG_ERR, "SIGSEGV/SIGABRT received!");
 	backtrace_symbols_fd(buffer, calls, 1);
 
 	str = backtrace_symbols(buffer, calls);
 	if (!str) {
-		syslog(LOG_ERR, "No backtrace strings found!");
+		tools_printlog(LOG_ERR, "No backtrace strings found!");
 		exit(EXIT_FAILURE);
 	}
 
 	for (i = 0; i < calls; i++)
-		syslog(LOG_ERR, "%s", str[i]);
+		tools_printlog(LOG_ERR, "%s", str[i]);
 	free(str);
 
-	exit(EXIT_FAILURE);
+	level = tools_log_get_level();
+	tools_log_set_level(LOG_DEBUG);
+	obj_print();
+	tools_log_set_level(level);
+	if (!obj_recovery())
+		exit(EXIT_FAILURE);
+}
+
+static int main_process(const char *config, int mode)
+{
+	objects_init();
+
+	if (nft_check_tables())
+		nft_reset();
+
+	loop_init();
+
+	if (config && config_file(config) != 0)
+		return EXIT_FAILURE;
+
+	if (tools_log_get_level() > NFTLB_LOG_LEVEL_DEFAULT)
+		obj_print();
+
+	obj_rulerize(OBJ_START);
+
+	if (mode == NFTLB_EXIT_MODE)
+		return EXIT_SUCCESS;
+
+	if (server_init() != 0) {
+		tools_printlog(LOG_ERR, "Cannot start server-ev: %s\n", strerror(errno));
+		return EXIT_FAILURE;
+	}
+
+	loop_run();
+
+	return EXIT_SUCCESS;
 }
 
 int main(int argc, char *argv[])
 {
+	int		c;
 	int		mode = NFTLB_SERVER_MODE;
 	int		run_mode = NFTLB_FG_MODE;
-	int		c;
-	int		loglevel = NFTLB_LOGLEVEL_DEFAULT;
+	int		loglevel = NFTLB_LOG_LEVEL_DEFAULT;
+	int		logoutput = NFTLB_LOG_OUTPUT_DEFAULT;
 	const char	*config = NULL;
+	pid_t	pid;
 
-	while ((c = getopt_long(argc, argv, "hl:c:k:ed6H:P:S", options, NULL)) != -1) {
+	while ((c = getopt_long(argc, argv, "hl:L:c:k:ed6H:P:Sm:", options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
 			print_usage(argv[0]);
@@ -123,6 +163,9 @@ int main(int argc, char *argv[])
 		case 'l':
 			loglevel = atoi(optarg);
 			break;
+		case 'L':
+			logoutput = atoi(optarg);
+			break;
 		case 'c':
 			config = optarg;
 			break;
@@ -148,9 +191,11 @@ int main(int argc, char *argv[])
 		case 'S':
 			serialize = 1;
 			break;
+		case 'm':
+			masquerade_mark = (int)strtol(optarg, NULL, 16);
+			break;
 		default:
-			fprintf(stderr, "Unknown option -%c\n", optopt);
-			syslog(LOG_ERR, "Unknown option -%c", optopt);
+			tools_printlog(LOG_ERR, "Unknown option -%c", optopt);
 			return EXIT_FAILURE;
 		}
 	}
@@ -158,50 +203,28 @@ int main(int argc, char *argv[])
 	if (signal(SIGINT, nftlb_sighandler) == SIG_ERR ||
 	    signal(SIGTERM, nftlb_sighandler) == SIG_ERR ||
 	    signal(SIGPIPE, SIG_IGN) == SIG_ERR ||
+	    signal(SIGABRT, nftlb_trace) == SIG_ERR ||
 	    signal(SIGSEGV, nftlb_trace) == SIG_ERR) {
-		fprintf(stderr, "Error assigning signals\n");
-		syslog(LOG_ERR, "Error assigning signals");
+		tools_printlog(LOG_ERR, "Error assigning signals");
 		return EXIT_FAILURE;
 	}
 
-	setlogmask(LOG_UPTO(loglevel));
+	tools_log_set_level(loglevel);
+	tools_log_set_output(logoutput);
 
-	if (nft_check_tables())
-		nft_reset();
-
-	objects_init();
-
-	loop_init();
-
-	if (config && config_file(config) != 0)
-		return EXIT_FAILURE;
-
-	if (loglevel > NFTLB_LOGLEVEL_DEFAULT)
-		obj_print();
-
-	obj_rulerize(OBJ_START);
-
-	if (mode == NFTLB_EXIT_MODE)
-		return EXIT_SUCCESS;
-
-	if (server_init() != 0) {
-		fprintf(stderr, "Cannot start server-ev: %s\n", strerror(errno));
-		return EXIT_FAILURE;
-	}
-
-	if ( run_mode ){
-		switch (fork()) {
-			case 0:
-				break;
-			case -1:
-				syslog(LOG_ERR, "Daemon mode aborted: %s", strerror(errno));
-				return EXIT_FAILURE;
-			default:
-				return EXIT_SUCCESS;
+	if (run_mode) {
+		pid = fork();
+		if (pid == -1) {
+			tools_printlog(LOG_ERR, "Daemon mode aborted: %s", strerror(errno));
+			return EXIT_FAILURE;
+		} else if (pid == 0) {
+			main_process(config, mode);
+		} else {
+			return EXIT_SUCCESS;
 		}
+	} else {
+		main_process(config, mode);
 	}
 
-	loop_run();
-
 	return EXIT_SUCCESS;
 }
